﻿using UniversityContracts.BindingModels;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using UniversityDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace UniversityDatabaseImplement.Implements
{
    internal class StudentStorage : IStudentStorage
    {
        public List<StudentViewModel> GetFullList()
        {
            using var context = new UniversityDatabase();
            return context.Students
                .Include(x => x.StudentExaminationResults)
                .ThenInclude(x => x.ExaminationResult)
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public List<StudentViewModel> GetFilteredList(StudentSearchModel model)
        {
            using var context = new UniversityDatabase();
            return context.Students
                .Include(x => x.StudentExaminationResults)
                .ThenInclude(x => x.ExaminationResult)
                .Where(x => (
                        (!model.Id.HasValue || x.Id == model.Id) &&
                        (string.IsNullOrEmpty(model.Name) || x.Name.Contains(model.Name))
                    )
                )
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public StudentViewModel? GetElement(StudentSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            return context.Students
                .Include(x => x.StudentExaminationResults)
                .ThenInclude(x => x.ExaminationResult)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && x.Name== model.Name) ||
                                     (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public StudentViewModel? Insert(StudentBindingModel model)
        {
            var newStudent = Student.Create(model);
            if (newStudent == null)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            context.Students.Add(newStudent);
            context.SaveChanges();
            return newStudent.GetViewModel;
        }

        public StudentViewModel? Update(StudentBindingModel model)
        {
            using var context = new UniversityDatabase();
            var student = context.Students.FirstOrDefault(x => x.Id == model.Id);
            if (student == null)
            {
                return null;
            }
            student.Update(model);
            context.SaveChanges();
            return student.GetViewModel;
        }
        public StudentViewModel? Delete(StudentBindingModel model)
        {
            using var context = new UniversityDatabase();
            var element = context.Students.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Students.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
