﻿using UniversityContracts.BindingModels;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using UniversityDatabaseImplement.Models;

namespace UniversityDatabaseImplement.Implements
{
    internal class UserStorage : IUserStorage
    {
        public List<UserViewModel> GetFullList()
        {
            using var context = new UniversityDatabase();
            return context.Users.Select(x => x.GetViewModel).ToList();
        }
        public List<UserViewModel> GetFilteredList(UserSearchModel model)
        {
            using var context = new UniversityDatabase();
            return context.Users
                .Where(x => (
                        (!model.Id.HasValue || x.Id == model.Id) &&
                        (string.IsNullOrEmpty(model.Login) || x.Login.Contains(model.Login)) &&
                        (string.IsNullOrEmpty(model.Password) || x.Password.Contains(model.Password))
                    )
                )
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public UserViewModel? GetElement(UserSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Login) && string.IsNullOrEmpty(model.Password) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            return context.Users
            .FirstOrDefault(x => (!string.IsNullOrEmpty(model.Login) && x.Login == model.Login) ||
                                 (!string.IsNullOrEmpty(model.Password) && x.Password == model.Password) ||
                                 (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public UserViewModel? Insert(UserBindingModel model)
        {
            var newUser = User.Create(model);
            if (newUser == null)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            context.Users.Add(newUser);
            context.SaveChanges();
            return newUser.GetViewModel;
        }
        public UserViewModel? Update(UserBindingModel model)
        {
            using var context = new UniversityDatabase();
            var user = context.Users.FirstOrDefault(x => x.Id == model.Id);
            if (user == null)
            {
                return null;
            }
            user.Update(model);
            context.SaveChanges();
            return user.GetViewModel;
        }
        public UserViewModel? Delete(UserBindingModel model)
        {
            using var context = new UniversityDatabase();
            var element = context.Users.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Users.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
